/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/>
 * ****************************************************************************/

/* 
 * This module provides 
 *   - class PDU that models a Dodwan-NAPI PDU 
 *   - a WebsocketConnection classthats offers methods and callbacks to 
 *        . handle a websocket connection 
 *        . transfer PDUs on this websocket
 */

console.log("Start module connection.js");
import * as util from "./util.js"
import WebSocket from "isomorphic-ws"



function log(msg) {
    util.serviceLog(null, 'napi_connection', msg);
}


/*
 * Class Pdu -------------------------------------------------
 * Command, response or notification
 */ 
export class Pdu {
    constructor() {}
    
}



/*
 * Class WebsocketConnectionService -----------------------------------------
 *
 */
export class WebsocketConnection {

    constructor() {
        this.socket = null;

        
        /** private **/
        this.reconnectNbTries = 0;
        
        this.reconnectNbTriesMax = 0;
        this.reconnectPeriod = 0;
        
        /*
         * Callback when the transmistter produced an error in sending or receiving
         */
        this.onSocketError = function () {log("Empty onSocketError()");};
        
        
        /*
         * Callback when the transmistter has opened the socket
         */
        this.onSocketOpen = function () {log("Empty onSocketOpen()");};
        
        /*
         * Callback when the transmistter produced an error in sending or receiving
         */
        this.onSocketClose = function () {log("Empty onSocketClose()");};

        /*
         * Callback when a PDU has been received
         */
        this.onReceivePdu = function () {log("Empty onreceivePdu()");};
    }
    ;
    
    
    
    /*
     * Setters for callbacks
     */
    
    setOnSocketError(f) { this.onSocketError = f; }
    setOnSocketOpen(f) { this.onSocketOpen = f; }
    setOnSocketClose(f) { this.onSocketClose = f; }
    setOnReceivePdu(f) { this.onReceivePdu = f; }

    
    
    /*
     * Opens the websocket  
     */
    connect(uri) {
        try {
            log("Connection to websocket server: "+uri)
            // Rq: a socket error is triggered when this fails
            this.socket = new WebSocket(uri);
        } catch (exception) {
            //console.error(exception);
            return;
        }
        
        this.socket.onerror =  (error) => {
            log("Connection failed");
            //console.error(error);
            ///this.reconnectLater(uri);
            this.onSocketError(error);
        };

        this.socket.onopen = (event) => {
            log("Connection established.");
            
            
            this.socket.onclose =  (event) => {
                log("Connection closed.");
                //this.reconnectLater(uri);
                this.onSocketClose();
            };

            if (typeof(window) === "undefined") {
                this.socket.onmessage =  (event) => {
                    //log("node data=" + event.data);
                    let pdu = JSON.parse(event.data);
                    //log("wsclient received PDU:" +JSON.stringify(pdu));
                    this.onReceivePdu(pdu);
                };
            }
            else {
                // when receiving a ws message
                this.socket.onmessage =  (event) => {

                    let reader = new FileReader();
                    //let blobReceived = new Blob();
                 
                    reader.onload = () => {
                        //log("wsclient received message (length="+reader.result.length+") : "+reader.result);
                        let pdu = JSON.parse(reader.result);
                        //log("[Connection] wsclient received PDU:" +JSON.stringify(pdu));
                        this.onReceivePdu(pdu);
                    };

                    // Read in the blob as a json string 
                    reader.readAsText(event.data);
                    //reader.readAsArrayBuffer(blobReceived);
                };
            };
            
            this.onSocketOpen();

        };
    }



    disconnect()  {
        this.socket.onclose = function() {};
        this.socket.close();
    }


    /*
     * Sends a PDU on the websocket
     * @param {Pdu} the PDU to be sent
     */
    sendPdu(pdu) {
        let serializedPdu = JSON.stringify(pdu);
        //log("[Connection] Sending PDU: "+serializedPdu);
        let buffer = new Uint8Array(util.stringToUtf8Array(serializedPdu));
        this.socket.send(buffer);
        return; 
    }
   
    
}


